# -*- coding: utf-8 -*-
# flake8: noqa
# pylint: skip-file
from ._sbn import query

__version__ = '0.0.1'

__plugin_author__ = 'arangb'
__plugin_description__ = 'Metadata for Italian books from opac.sbn.it'
__plugin_homepage__ = 'github.com/arangb/isbnlib-sbn'
__plugin_service_language__ = 'it'
__plugin_service_name__ = 'sbn'
__plugin_type__ = 'metadata'

